/**
 * @package xbeemicrolib
 * @filename xbeemicrolib.h
 * 
 * @author ysuga (ysuga.net)
 * @copyright ysuga.net 2011 allrights reserved
 * @license GPL
 */

#ifndef XBEEMICROLIB_HEADER_INCLUDED
#define XBEEMICROLIB_HEADER_INCLUDED


/***********************************/
/* Error Codes                     */
/***********************************/
#define ERR_TIMEOUT -101
#define ERR_INVALID -102
#define ERR_CHECKSUM -103
#define ERR_UART_BUSY -104
#define ERR_INVALID_HOST -105
#define ERR_TRANSMIT_BUSY -106
#define ERR_RX_EMPTY  -107


/**
 * @brief Initialization Function
 *  Here, settings below are configured automatically.
 *  1. AP=2, 
 *  2. NI (Network Identifier String)
 *  3. SC (Scan PAN Mask)
 *  4. PANID (64bit PAN ID)
 */
void XBee_setup(void);


#define XBEE_MODE_STARTUP     1
#define XBEE_MODE_INITIALIZED 2
/**
 * @brief get Mode of XBee module
 *
 * @return XBEE_MODE_STARTUP: just initialized but not in use. XBEE_MODE_INITIALIZED: initialized and can be used.
 */
int XBee_getMode();


/**
 * @brief polling message from XBee module.
 * This function must be periodically called in your application. If data is received, automatically called onRxPacket function.
 * The onRxPacket function can be registered by your application code. See XBee_registerOnRxPacket function.
 */
int XBee_polling(void);




/***********************************/
/* Host Record                     */
/***********************************/
#define XBEE_HOST_UNKNOWN       101
#define XBEE_HOST_KNOWN         0
#define XBEE_HOST_CANNOTRESOLVE 102

/**
 * @brief Host Record Struct
 *
 */
typedef struct HostRecord_t {
	/**
	 * @brief Network Identifier String. 
	 * Use XBee_resolve function. XBee can resolve network address from this string.
	 */
	char* networkIdentifierString;

	/**
	 * @brief this indicate the host record is ready for use (or not).
	 */
	unsigned char status;

	/**
	 * @brief 64 bit network address. Actually this value is configured by factory. 
	 * this value is equal to the serial number of XBee.
	 */
	unsigned char addr64[8];

	/**
	 * @brief 16 bit network address. This value is given when the XBee device joins to the PAN (personal area network).
	 */
	unsigned char addr16[2];
}XBeeHostRecord;

/**
 * @brief Use this macro for XBeeHostRecord initialization.
 */
#define HOST_RECORD_INITIALIZER(str) {str, XBEE_HOST_UNKNOWN, {0,}, {0,}}

/**
 * @brief Use this macro for XBeeHostRecord Initialization as Coordinator
 * Coordinator address is special (zero) so you do not have to resove it.
 */
#define HOST_RECORD_COORDINATOR {"", XBEE_HOST_KNOWN, {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, {0xFF, 0xFE}}

/**
 * @brief resolve network address from node identifier string
 *
 * @param pHost host record.
 * @return zero if success.
 */
int XBee_resolve(XBeeHostRecord* pHost);

/**
 * @brief wait until resolve is done
 *
 * @return zero if success.
 */
unsigned char XBee_waitResolveDone(void);






/**
 * @brief packet transmit
 * This function can not indicate the delivery status. This function just do request transmission to the XBee module.
 *
 * @param pDst destination host record (this should be ready for transmit. see XBee_resolve)
 * @param dat  data buffer
 * @param size size of data
 * @return zero if success. ERR_UART_BUSY (Tx Buffer is busy). ERR_INVALID_HOST (host record is not ready for use)
 */
int XBee_transmit(const XBeeHostRecord* pDst, const unsigned char* dat, const unsigned char size);

/*******************/
/* Delivery Status */
/*******************/
#define XBEE_DELIVERY_STATUS_SUCCESS             0x00
#define XBEE_DELIVERY_STATUS_CCAFAILURE          0x02
#define XBEE_DELIVERY_STATUS_INVALID_DESITATION  0x15
#define XBEE_DELIVERY_STATUS_NETWORKACKFAILURE   0x21
#define XBEE_DELIVERY_STATUS_NOTJOINEDTONETWORK  0x22
#define XBEE_DELIVERY_STATUS_SELFADDRESSED       0x23
#define XBEE_DELIVERY_STATUS_ADDRESSNOTFOUND     0x24
#define XBEE_DELIVERY_STATUS_ROUTENOTFOUND       0x25
#define XBEE_DELIVERY_STATUS_DATAPAYLOADTOOLARGE 0x74

/**
 * @brief wait transmit is done
 *
 * @return success:XBEE_DELIVERY_STATUS_SUCCESS, error see XBEE_DELIVERY_STATUS_***
 */
unsigned char XBee_waitTransmitCommandDone(void);

/**
 * @brief this function register an event handler on packet received.
 * 
 * @param onRxPacket pointer to function. the function must be like void(*onRxPacket)(const char* addr64, const char* addr16, const char* dat, const unsigned short size);
 */
void XBee_registerOnRxPacket(void(*onRxPacket)(const char* addr64, const char* addr16, const char* dat, const unsigned short size));



#endif // #ifndef XBEEMICROLIB_HEADER_INCLUDED
